@@ -4,7 +4,7 @@ import multiprocessing
 import os
 import threading
 import time
-from concurrent.futures import ProcessPoolExecutor, wait
+from concurrent.futures import ProcessPoolExecutor, wait, FIRST_EXCEPTION
 from typing import Dict, Optional, Union, List
 
 import boto3
@@ -223,7 +223,11 @@ class CloudClient:
             if len(batch) > 0:
                 futures.append(executor.submit(CloudClient.file_upload_handler, (Job(count, identifier, batch))))
 
-            wait(futures)
+            done, not_done = wait(futures, return_when=FIRST_EXCEPTION)
+            for future in not_done:
+                future.cancel()
+            for future in done:
+                future.result()
 
         return os.path.join("/", bucket, upload_prefix, os.path.basename(file_path))
 
@@ -264,10 +268,9 @@ class CloudClient:
             bucket: str,
             key: str,
             fileobj
-    ) -> str:
+    ):
         client = CloudClient._get_client(identifier, False)
-        client.download_fileobj(bucket, key, fileobj)
-        return
+        client.download_fileobj(Bucket=bucket, Key=key, Fileobj=fileobj)
 
     @staticmethod
     def download_files(
@@ -322,14 +325,17 @@ class CloudClient:
             if len(batch) > 0:
                 futures.append(executor.submit(CloudClient.file_download_handler, (Job(count, identifier, batch))))
 
-            wait(futures)
+            done, not_done = wait(futures, return_when=FIRST_EXCEPTION)
+            for future in not_done:
+                future.cancel()
+            for future in done:
+                future.result()
 
         return os.path.join(file_path, prefix)
 
     @staticmethod
     def file_download_handler(job: Job):
         client = CloudClient._get_client(job.identifier)
-
         for item in job.batch:
             client.download_file(item.bucket, item.key, item.file_name)
 
