@@ -2,15 +2,16 @@ import json
 import logging
 import os.path
 import time
-from typing import List, Dict, Optional, Any, TYPE_CHECKING
+from typing import List, Dict, Optional, Any, TYPE_CHECKING, Set
 
 from s3transfer.constants import GB
 from tqdm import tqdm
+from urllib.parse import urlparse
 
 from metaloop.client.cloud_storage import CloudConfig, CloudClient, Item
 from metaloop.client.download_helper import DownloadHelper
 from metaloop.utils.file_helper import get_file_type, FileTypeZip
-from metaloop.exception import ResourceNotExistError, InvalidParamsError
+from metaloop.exception import ResourceNotExistError, InvalidParamsError, InternalServerError
 
 if TYPE_CHECKING:
     from metaloop.client.mds import MDS
@@ -135,8 +136,9 @@ class Dataset(DatasetMeta):
         version = info["version"]
 
         if inherited_version_number is not None and inherited_version_number > -1:
-            self._inherit_import(dataset_id, inherited_version_number)
             self.checkout(inherited_version_number)
+            if self._data_count > 0:
+                self._inherit_import(dataset_id, inherited_version_number)
 
         self._versions[version] = dataset_id
         self._id = dataset_id
@@ -239,6 +241,9 @@ class Dataset(DatasetMeta):
         if not file_path:
             raise InvalidParamsError(param_name="file_path", param_value=file_path)
 
+        if storage_type not in {"local", "cos"}:
+            raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
+
         print(f"upload file to server, please wait...")
 
         if storage_type == "cos":
@@ -246,13 +251,13 @@ class Dataset(DatasetMeta):
                 file_type = get_file_type(file_path)[1]
                 if file_type == FileTypeZip:
                     raise InvalidParamsError(message="Tencent COS uploading do not support compressed file")
-                upload_id = self._upload_file(storage_type, file_path)
+                upload_id = self._upload_file(file_path, storage_type)
             else:
                 upload_id = self._upload_files(file_path, storage_type, self._id)
             task_type = "cos"
 
         elif os.path.isdir(file_path) and os.path.getsize(file_path) > 1 * GB:
-            upload_id = self._upload_file(storage_type, file_path)
+            upload_id = self._upload_file(file_path, storage_type)
             task_type = "object_storage"
 
         elif os.path.isdir(file_path):
@@ -300,6 +305,9 @@ class Dataset(DatasetMeta):
                 2. "external": Tencent Cloud Object Storage (COS).
 
         """
+        if storage_type not in {"local", "external"}:
+            raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
+
         export_type = "annotation_uri" if self._annotation_status == 2 else "uri_json_standard"
 
         post_data: Dict[str, Any] = {
@@ -330,6 +338,8 @@ class Dataset(DatasetMeta):
         for obj in tqdm(catalog):
             count += 1
             ex = False
+            image_path = obj["image_path"] if "image_path" in obj else ""
+
             if "external_url_image" in obj:
                 # /{bucket}/{path}/{to}/{object}.ext
                 url = obj["external_url_image"]
@@ -339,7 +349,6 @@ class Dataset(DatasetMeta):
                     continue
 
                 s3_config = self._get_cloud_storage_config(items[0])
-                image_path = obj["image_path"] if "image_path" in obj else ""
 
                 file_name = download_helper.download_s3_file(
                     self._download_file,
@@ -358,7 +367,8 @@ class Dataset(DatasetMeta):
                 file_name = download_helper.download_http_file(
                     self._download_http_file,
                     url=url,
-                    file_path=file_path
+                    file_path=file_path,
+                    image_path=image_path
                 )
                 obj["url_image"] = file_name
 
@@ -396,7 +406,7 @@ class Dataset(DatasetMeta):
     ) -> None:
         try:
             version_id = self._versions[version_number]
-        except IndexError as error:
+        except KeyError as error:
             raise ResourceNotExistError(resource="version", identification=version_number) from error
 
         post_data: Dict[str, Any] = {
@@ -419,6 +429,8 @@ class Dataset(DatasetMeta):
         while True:
             info = self._x_api.get_import_status(self._id)
             progress = info["progress"]
+            if progress == -1:
+                raise InternalServerError(message=info["message"])
             if pbar:
                 pbar.update(progress - last_progress)
             if progress >= 100:
@@ -437,6 +449,8 @@ class Dataset(DatasetMeta):
         while True:
             info = self._x_api.get_export_status(self._id)
             progress = info["progress"]
+            if progress == -1:
+                raise InternalServerError(message=info["message"])
             if pbar:
                 pbar.update(progress - last_progress)
             if progress >= 100:
@@ -473,40 +487,47 @@ class Dataset(DatasetMeta):
     def _download_file(
             **kwargs
     ) -> (str, Item):
-        index = kwargs["url"].strip("/").index("/")
-        key = kwargs["url"][index + 2:]
-
-        file_path = kwargs["file_path"]
-        image_path = kwargs["image_path"]
-
-        if not file_path:
-            file_path = "abaddon"
-
-        if image_path:
-            image_path = image_path.strip("/")
-            file_name = os.path.join(file_path, image_path)
-        else:
-            file_name = os.path.join(file_path, key)
-
-        if not os.path.exists(os.path.dirname(file_name)):
-            os.makedirs(os.path.dirname(file_name), 0o0755)
-
-        return kwargs["identifier"], Item(kwargs["bucket"], key, file_name, image_path if image_path else key)
+        key, file_name, obj_url = Dataset._get_download_info(
+            kwargs["url"],
+            kwargs["file_path"],
+            kwargs["image_path"]
+        )
+        return kwargs["identifier"], Item(kwargs["bucket"], key, file_name, obj_url)
 
     @staticmethod
     def _download_http_file(
             **kwargs
     ) -> Item:
-        url = kwargs["url"]
-        file_path = kwargs["file_path"]
+        _, file_name, obj_url = Dataset._get_download_info(
+            urlparse(kwargs["url"]).path,
+            kwargs["file_path"],
+            kwargs["image_path"]
+        )
+        return Item("", kwargs["url"], file_name, obj_url)
 
-        index = url.strip("http://").index("/")
-        file_name = os.path.join(file_path, url[index + 8:])
+    @staticmethod
+    def _get_download_info(
+            url: str,
+            file_path: str,
+            image_path: str
+    ) -> (str, str, str):
+        index = url.strip("/").index("/")
+        key = url[index + 2:]
+
+        if image_path:
+            file_name = os.path.join(file_path, image_path.strip("/"))
+        else:
+            file_name = os.path.join(file_path, url.strip("/"))
+
+        if not file_path:
+            file_name = os.path.join("abaddon", file_name)
 
         if not os.path.exists(os.path.dirname(file_name)):
             os.makedirs(os.path.dirname(file_name), 0o0755)
 
-        return Item("", url, file_name, url[index + 8:])
+        obj_url = file_name.strip(file_path).strip("/")
+
+        return key, file_name, obj_url
 
     def _download_files(
             self,
